#!/bin/sh
# exb2metadata.sh -- get corpus-level metadata from an EXB file
# Andreas Nolda 2019-04-16

set -e

progname=$(basename $0)
progpath=$(dirname $(realpath $0))

EXMARALDADULKODIR=$progpath
SAXONDIR=$progpath

JAVA_OPTIONS="-Xmx512M"

CP_JAVA=$SAXONDIR/saxon9.jar

saxon="java $JAVA_OPTIONS -classpath $CP_JAVA net.sf.saxon.Transform -expand:off"

usage="\
Usage: $progname [-c] [--corpus <directory relative to the EXB file>]
                       <EXB file>
       $progname [-h] [--help]"
version=2.0

dir=
file=

if [ $# -eq 0 ] ; then
  echo "$usage" 1>&2
  exit 1
else
  while [ $# -gt 0 ]; do
    case "$1" in
      -c | --corpus)
        shift;
        dir=$1;;
      -h | --help)
        echo "\
$progname, version $version -- get corpus-level metadata from an EXB file.";
        echo "$usage";
        exit 0;;
      -)
        file="$1";;
      -*)
        echo "$usage" 1>&2;
        exit 1;;
      *)
        file="$1";;
    esac
    shift
  done
fi
if [ -z "$file" ]; then
  echo "$usage" 1>&2
  exit 1
fi

run_saxon() {
  $saxon "$1" $EXMARALDADULKODIR/share/exb2metadata.xsl $2
}

if [ -n "$dir" ]; then
  run_saxon "$file" corpus="$dir"
else
  run_saxon "$file"
fi

exit 0
